module switch.

import popeyesig, javasig, typechecker, spinach, javaspi.

% Interpreter stuff
%%%%%%%%%%%%%%%%%%%

% Lambda Prolog declarations
%
type switch 	class.

type sw 	ref.

type get	ref -> ref -> cmd.
type set 	ref -> ref -> cmd.

type toggle	ref -> cmd.

% Module per se
%
popmod ( switch ) (
	 
	S\

	poploc "sw" switch sw sw\

	( popcon ( (pv sw null) :: nil )
		 (
		  ( poppi "old" Old\
		    cons S (switch) >- rw sw Old (pbool poptrue) ) ::

		  ( poppi "val" Val\
		    get S Val >- rw sw Val Val	 	    ) ::

		  ( poppi "old" Old\
		    poppi "val" Val\
		    set S Val >- rw sw Old Val              ) ::

		  ( toggle S >- rw sw (pbool poptrue) (pbool poptrue) +> set S (pbool popfalse) 
		                                		      |> set S (pbool poptrue)  ) ::

		  nil
		 )
	)
).

% Typechecker stuff
%%%%%%%%%%%%%%%%%%%%

% Extension
%
extends switch object.

% otype switch class.
%
tcheck Theta switch Cont tmod TC :- 
	popmod switch M,
	pi C\ tcheck ((tp C (tref switch))::Theta) (M C) Cont tprog TC.

tcheck Theta (new switch P) Cont tcmd TC :-
	pi C\ tcheck ((tp C (tref switch))::Theta) (P C) Cont tcmd TC.

tcheck Theta (cons S switch) Cont theadin TC :-
	tcheck Theta S nil (tref switch) nil,						
	tcheck Theta none Cont dot TC.	

tcheck Theta (cons S switch) Cont theadout TC :-
	tcheck Theta none Cont dot TC.

tcheck Theta (super C switch) Cont tcmd TC :-
	tcheck Theta none Cont dot TC.

% Private data
%
tcheck Theta (ppriv switch sw) Cont popbool TC :- 
	tcheck Theta none Cont dot TC.			% ptype sw switch boolean.

% No constants here
%

% Methods
%
% mtype get	 out 	ref switch -> boolean -> cmd.
%
tcheck Theta (get S V) Cont tcmd TC :- 
	tcheck Theta S nil (tref switch) nil,
	tcheck Theta V nil dot nil,
	tcheck ((tp V popbool)::Theta) none Cont dot TC.	% read	

tcheck Theta (get S V) Cont ttest TC :-
	tcheck Theta S nil (tref switch) nil,
	tcheck Theta V nil popbool nil,
	tcheck Theta none Cont dot TC.				% test

tcheck Theta (get S V) Cont theadin TC :-
	tcheck Theta S nil (tref switch) nil,
	tcheck Theta none Cont dot TC.				% headin

tcheck Theta (get S V) Cont theadout TC :-
	tcheck Theta V nil popbool nil,
	tcheck Theta none Cont dot TC.				% headout
 
% mtype set	in 	ref switch -> boolean -> cmd.
%
tcheck Theta (set S V) Cont tcmd TC :- 
	tcheck Theta S nil (tref switch) nil,
	tcheck Theta V nil popbool nil,
	tcheck Theta none Cont dot TC.				% write	

tcheck Theta (set S V) Cont theadin TC :-
	tcheck Theta S nil (tref switch) nil,
	tcheck ((tp V popbool)::Theta) none Cont dot TC.	% headin

tcheck Theta (set S V) Cont theadout TC :-
	tcheck Theta none Cont dot TC.				% headout

% mtype toggle	in	ref switch -> cmd.
%
tcheck Theta (toggle S) Cont tcmd TC :- 
	tcheck Theta S nil (tref switch) nil,
	tcheck Theta none Cont dot TC.				% write	

tcheck Theta (toggle S) Cont theadin TC :-
	tcheck Theta S nil (tref switch) nil,
	tcheck Theta none Cont dot TC.				% headin

tcheck Theta (toggle S) Cont theadout TC :-
	tcheck Theta none Cont dot TC.				% headout

% Spinacher stuff
%%%%%%%%%%%%%%%%%

% otype switch class.
%
spinach Theta Mu (tref switch) "Switch".

spinach Theta Mu switch Java :-
	popmod switch P,
	extends switch O,
	spinach nil nil (tref O) SO,
	pi C\ spinach ((tp C (tref switch))::Theta) ((np C "this")::Mu) (P C) JP,
	Java is "public class" ^ "Switch" ^ " extends " ^ SO ^ "{\n" ^ JP ^ "}\n".

spihead Theta Mu (cons W (switch)) nil Java "" :-
	Java is "Switch(){\n".

spinach Theta Mu (popnew switch S P) Java :-
	pi O\ spinach ((tp O (tref switch))::Theta) ((np O S)::Mu) (P O) JP,
	spinach nil nil (tref switch) JS,
	Java is JS ^ " " ^ S ^ " = new " ^ JS ^ "();\n" ^ JP.	

spinach Theta Mu (super C (switch)) Java :-
	Java is "super(" ^ ");\n".

% mtype get	out 	ref switch -> boolean -> cmd.
%
spihead Theta Mu (get S V) nil Java Jreturn :-
	spinach nil nil popbool STV,
	Java is "public " ^ STV  ^" get(){\n",
	spinach Theta Mu V SV, 
	Jreturn is "return " ^ SV ^ ";\n".

spinach Theta Mu (get S V) Java :-
	spinach Theta Mu S SS,
	spinach Theta Mu V SV,
	Java is SV ^ "=" ^ SS ^ ".get();". 	

spitest Theta Mu (get S V) Java :-
	spinach Theta Mu S SS,
	spinach Theta Mu V SV,
	Java is SV ^ "==" ^ SS ^ ".get()".

% mtype set 	in 	ref switch -> boolean -> cmd.
%
spihead Theta Mu (set S V) (V::nil) Java "" :-
	spinach nil nil popbool STV,
	spinach Theta Mu V SV,
	Java is "public void set( " ^ STV ^ " " ^ SV ^ " ){\n".

spinach Theta Mu (set S V) Java :-
	spinach Theta Mu S SS,
	spinach Theta Mu V SV,
	Java is SS ^ ".set(" ^ SV ^ ");\n". 

% mtype toggle  in 	ref switch -> cmd.
%
spihead Theta Mu (toggle S) nil Java "" :-
	Java is "public void toggle(){\n".

spinach Theta Mu (toggle S) Java :-
	spinach Theta Mu S SS,
	Java is SS ^ ".toggle();\n".




